import { Observable, forkJoin } from 'rxjs';
import { take, concatMap } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';
import {
  Resolve,
  ActivatedRouteSnapshot,
  RouterStateSnapshot
} from '@angular/router';

import { Store } from '@ngrx/store';

import { StatusCode } from '@ucap/api';
import { NativeService } from '@ucap/native';
import { SSOMode } from '@ucap/protocol-authentication';

import { LogService } from '@ucap/ng-logger';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { PublicApiService } from '@ucap/ng-api-public';
import { ExternalApiService } from '@ucap/ng-api-external';
import { ProtocolService } from '@ucap/ng-protocol';
import { InnerProtocolService } from '@ucap/ng-protocol-inner';
import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';

import { CompanyActions } from '@ucap/ng-store-organization';
import {
  ConfigurationActions,
  LoginActions
} from '@ucap/ng-store-authentication';

import { AppAuthenticationService } from '@app/services/app-authentication.service';

@Injectable()
export class AppAuthenticationResolver implements Resolve<void> {
  constructor(
    private publicApiService: PublicApiService,
    private externalApiService: ExternalApiService,
    private protocolService: ProtocolService,
    private innerProtocolService: InnerProtocolService,
    private authenticationProtocolService: AuthenticationProtocolService,
    private store: Store<any>,
    private appAuthenticationService: AppAuthenticationService,
    private logService: LogService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}

  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): void | Observable<void> | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        const loginSession = this.appAuthenticationService.loginSession;
        const networkInfo = await this.nativeService.getNetworkInfo();
        const localIp =
          !!networkInfo && 0 < networkInfo.length && !!networkInfo[0].ip
            ? networkInfo[0].ip
            : '';
        const localMac =
          !!networkInfo && 0 < networkInfo.length && !!networkInfo[0].mac
            ? networkInfo[0].mac
            : '';

        this.store.dispatch(
          CompanyActions.companies({
            req: { companyGroupCode: loginSession.companyGroupCode }
          })
        );

        forkJoin([
          this.publicApiService
            .versionInfo2({
              deviceType: loginSession.deviceType,
              desktopType: loginSession.desktopType,
              companyGroupType: loginSession.companyGroupType,
              companyCode: loginSession.companyCode,
              loginId: loginSession.loginId
            })
            .pipe(take(1)),
          this.externalApiService
            .urlInfo({
              deviceType: loginSession.deviceType,
              loginId: loginSession.loginId
            })
            .pipe(take(1))
        ])
          .pipe(take(1))
          .subscribe(
            ([versionInfo2Res, urlInfoRes]) => {
              if (
                StatusCode.Fail === versionInfo2Res.statusCode ||
                StatusCode.Fail === urlInfoRes.statusCode
              ) {
                if (StatusCode.Fail === versionInfo2Res.statusCode) {
                  this.store.dispatch(
                    ConfigurationActions.versionInfo2Failure({
                      error: versionInfo2Res.errorMessage
                    })
                  );

                  reject(versionInfo2Res.errorMessage);
                }

                if (StatusCode.Fail === urlInfoRes.statusCode) {
                  this.store.dispatch(
                    ConfigurationActions.urlInfoFailure({
                      error: urlInfoRes.errorMessage
                    })
                  );

                  reject(urlInfoRes.errorMessage);
                }

                return;
              }

              this.store.dispatch(
                ConfigurationActions.versionInfo2Success({
                  res: versionInfo2Res
                })
              );

              this.store.dispatch(
                ConfigurationActions.urlInfoSuccess({
                  res: urlInfoRes
                })
              );

              this.protocolService
                .connect(versionInfo2Res.serverIp)
                .pipe(
                  take(1),
                  concatMap(() => {
                    return this.innerProtocolService.conn({}).pipe(
                      take(1),
                      concatMap((connRes) => {
                        return this.authenticationProtocolService
                          .login({
                            loginId: loginSession.loginId,
                            loginPw: loginSession.loginPw,
                            deviceType: loginSession.deviceType,
                            deviceId: ' ',
                            token: '',
                            localeCode: loginSession.localeCode,
                            pushId: ' ',
                            companyCode: loginSession.companyCode,
                            passwordEncodingType: 1,
                            clientVersion: '',
                            reconnect: false,
                            ip: localIp,
                            hostName: localMac,
                            ssoMode: SSOMode.AUTH,
                            userSpecificInformation: 'PRO_000482',
                            andId: '',
                            andPushRefreshYn: ''
                          })
                          .pipe(take(1));
                      })
                    );
                  })
                )
                .subscribe(
                  (loginRes) => {
                    this.store.dispatch(
                      LoginActions.loginSuccess({
                        res: loginRes,
                        loginSession
                      })
                    );
                    resolve();
                  },
                  (error) => {
                    reject(error);
                  }
                );
            },
            (error) => {
              reject(error);
            }
          );
      } catch (error) {
        reject(error);
      }
    });
  }
}
